#!/usr/bin/env python
"""
An autonomous drive node that uses native_ros imitation of rospy.
It subscribes to simulated LaserScan messages and publishes Twist messages.
"""

from time import sleep

import native_rospy as rospy

# --- Minimal Message Definitions ---
# These classes mimic the ROS message types used in the original example.


class Twist:
    def __init__(self):
        # Using simple objects for linear and angular components
        self.linear = type("Vector", (), {"x": 0.0})()
        self.angular = type("Vector", (), {"z": 0.0})()

    def __str__(self):
        return "Twist(linear.x={:.2f}, angular.z={:.2f})".format(self.linear.x, self.angular.z)


class LaserScan:
    def __init__(self, ranges):
        self.ranges = ranges


# --- Autonomous Driver Class ---


class AutonomousDriver:
    def __init__(self):
        # Create a publisher for driving commands on the "/cmd_vel" topic.
        self.cmd_vel_pub = rospy.Publisher("/cmd_vel", Twist)
        # Subscribe to LaserScan messages on the "/scan" topic.
        self.scan_sub = rospy.Subscriber("/scan", LaserScan, self.scan_callback)
        # Initialize the Twist command to be published.
        self.twist = Twist()
        # Set safe distance threshold (in meters)
        self.safe_distance = 1.0
        self.obstacle_detected = False

    def scan_callback(self, data):
        """
        Process incoming LaserScan messages.
        Checks a small window in the center of the scan data.
        """
        num_samples = len(data.ranges)
        # Take a window around the center (adjust slice size as needed)
        front_ranges = data.ranges[num_samples // 2 - 5 : num_samples // 2 + 5]
        # Filter out any 'inf' values
        valid_ranges = [r for r in front_ranges if r < float("inf")]
        min_range = min(valid_ranges) if valid_ranges else float("inf")
        self.obstacle_detected = min_range < self.safe_distance

    def run(self):
        """
        Main loop: publish velocity commands based on sensor input.
        """
        rate = rospy.Rate(10)  # 10 Hz loop rate
        while not rospy.is_shutdown():
            if self.obstacle_detected:
                # Obstacle detected: stop forward motion and rotate.
                self.twist.linear.x = 0.0
                self.twist.angular.z = 0.5
                rospy.loginfo("Obstacle detected - rotating")
            else:
                # No obstacle: drive forward.
                self.twist.linear.x = 0.2
                self.twist.angular.z = 0.0
                rospy.loginfo("Path clear - moving forward")
            # Publish the Twist command.
            self.cmd_vel_pub.publish(self.twist)
            rate.sleep()


# --- Simulation of LaserScan Publishing ---
# Since this is a native imitation with no underlying middleware, we simulate
# laser scan messages being published periodically.


def simulate_scan():
    """
    Periodically publish simulated LaserScan messages with random data.
    Sometimes an obstacle (a short distance reading) is inserted.
    """
    import random

    from native_rospy import get_default_middleware

    mw = get_default_middleware()
    while True:
        # Create a fake scan with 100 range samples.
        ranges = [random.uniform(0.5, 2.0) for _ in range(100)]
        # 30% chance to simulate an obstacle in the middle of the scan.
        if random.random() < 0.3:
            for i in range(45, 55):
                ranges[i] = random.uniform(0.0, 0.8)
        scan_msg = LaserScan(ranges)
        mw.publish("/scan", scan_msg)
        sleep(0.1)


if __name__ == "__main__":
    rospy.init_node("autonomous_drive_node")
    auto_driver = AutonomousDriver()

    try:
        # Start the scan simulation in a background thread.
        import threading

        sim_thread = threading.Thread(target=simulate_scan, daemon=True)
        sim_thread.start()
        # Run the autonomous drive loop.
        auto_driver.run()
    except KeyboardInterrupt:
        pass
