from queue import Queue

import cv2
from loguru import logger
from PIL import Image
from rosbags.image import message_to_cvimage

import native_rospy as rospy

try:
    from sensor_msgs.msg import Image as ROSImage
except ImportError:
    ROSImage = None


def process_image(msg) -> Image.Image:
    """
    Process the ROS Image message and convert it into a PIL.Image.
    """
    cv_image = message_to_cvimage(msg)
    if cv_image is None:
        return None

    # Convert from BGR (OpenCV format) to RGB (PIL format)
    rgb_image = cv2.cvtColor(cv_image, cv2.COLOR_BGR2RGB)
    # Create a PIL image from the numpy array
    pil_image = Image.fromarray(rgb_image)

    # Optionally, resize the image (e.g., from 1280x720 to 672x378)
    # pil_image = pil_image.resize((672, 378))

    return pil_image


class CamViewer:
    def __init__(self):
        # Queue to temporarily hold incoming images (up to 3 images)
        self.image_queue = Queue(maxsize=3)
        # Subscribe to the ROS image topic. Replace the topic name and type if necessary.
        self.sub = rospy.Subscriber("/rgb_front/compressed", ROSImage, self.callback)
        # Timer to trigger image saving periodically.
        # The timer period is set to 20 seconds; adjust as needed.
        self.timer = rospy.Timer(20, self.show_image)
        self.idx = 0

    def callback(self, msg):
        """
        Callback function for the ROS subscriber.
        Converts the incoming message to a PIL image and adds it to the queue.
        """
        # If the queue is full, remove the oldest image.
        if self.image_queue.full():
            self.image_queue.get()
        if msg is not None:
            self.image_queue.put(msg)
        else:
            logger.warn("Processed image is None. Skipping.")

    def show_image(self):
        """
        Timer callback that saves an image from the queue.
        """
        if not self.image_queue.empty():
            msg = self.image_queue.get()
            image = process_image(msg)

            self.idx += 1
            filename = f"image_{self.idx}.png"
            try:
                image.save(filename)
                logger.info(f"Saved image: {filename}")
            except Exception as e:
                logger.error(f"Failed to save image: {filename}, {e}")
        else:
            logger.info("Image queue is empty, nothing to save.")


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("mcap", type=str, help="Path to the mcap file")
    args = parser.parse_args()

    rospy.init_node("cam_viewer", anonymous=True)
    try:
        cv_viewer = CamViewer()
        # rospy.spin()
        rospy.native.mcap_replay(args.mcap, topics=["/rgb_front/compressed"], show_pbar=True)
    except rospy.ROSInterruptException:
        pass
